#!/usr/bin/env python
# coding=utf-8
'''
Author: waton
Date: 2024-07-30 13:38:57
LastEditTime: 2024-07-30 18:18:33
FilePath: \my_plugin\plugin\file_utils.py
Description: 
'''
#!/usr/bin/env python
# coding=utf-8



import os, json, yaml, xlrd, csv
from abc import ABCMeta, abstractmethod
from typing import Dict, List, Tuple, Union, Any
from openpyxl import workbook


class FileUtils(metaclass=ABCMeta):
    @abstractmethod
    def read(self, file_path: str):
        """
        抽象方法，用于读取文件
        :param file_path: 文件路径
        :return: 文件内容，可能是字符串或字节数组
        """
        pass

    @abstractmethod
    def write(self, file_path: str, content: Union[List, Dict, Tuple, str, bytes], mode: str = 'w'):
        """
        抽象方法，用于写入文件
        :param file_path: 文件路径
        :param content: 要写入的内容，可以是列表、字典、元组、字符串或字节数组
        :param mode: 写入模式，默认为'w'，表示覆盖写入
        """
        pass

class JsonFileUtils(FileUtils):
    def read(self, file_path):
        with open(file_path, 'r', encoding='utf-8') as f:
            return json.load(f)

    def write(self, file_path, content: Dict, mode='w'):
        """
        将内容写入json文件

        Args:
            file_path (str): 文件路径
            content (Dict): 要写入的内容
            mode (str, optional): 写入模式，默认为'w'，表示覆盖写入. Defaults to 'w'.
        """
        with open(file_path, 'w', encoding='utf-8') as f:
            json.dump(content, f, indent=4, ensure_ascii=False)

class YamlFileUtils(FileUtils):
    def read(self, file_path):
        with open(file_path, 'r', encoding='utf-8') as f:
            return yaml.load(f, Loader=yaml.FullLoader)

    def write(self, file_path, content: Union[Dict, List, Tuple, str], mode='w'):
        """
        将数据写入到yaml文件中

        Args:
            file_path: 文件路径
            content: 要写入的数据，可以是字典、列表、元组等
            mode: 写入模式，默认为'w'
        """
        with open(file_path, mode=mode, encoding='utf-8') as f:
            yaml.dump(content, f, allow_unicode=True)  # 写入的中文不进行转码

class ExcelFileUtils(FileUtils):
    def read(self, file_path):
        with xlrd.open_workbook(file_path) as f:
            sheet = f.sheet_by_index(0)
            rows = sheet.nrows
            cols = sheet.ncols
            data = []
            for row in range(rows):
                row_data = []
                for col in range(cols):
                    row_data.append(sheet.cell_value(row, col))
                data.append(row_data)
            return data

            

    def write(self, file_path, content: list[list]):
        """
        将数据写入 Excel 文件

        Args:
            file_path (str): 文件保存路径     
            content (list[list]): 要写入的数据，二维列表
        """
        wb = workbook.Workbook()
        sheet = wb.active

        for row, row_data in enumerate(content):
            for col, value in enumerate(row_data):
                sheet.cell(row=row + 1, column=col + 1, value=value)

        wb.save(file_path)

class CSVFileUtils(FileUtils):
    def read(self, file_path):
        with open(file_path, 'r', encoding='utf-8') as f:
            reader = csv.reader(f)
            data = [row for row in reader]
            return data

    def write(self, file_path, content: list[list], mode='w'):
        """
        将数据写入 CSV 文件

        Args:
            file_path (str): 文件保存路径
            content (list[list]): 要写入的数据，二维列表
            mode (str): 文件打开模式，默认为 'w'
        """
        with open(file_path, mode=mode, encoding='utf-8', newline='') as f:
            writer = csv.writer(f)
            writer.writerows(content)
            
if __name__ == '__main__':
    # json_file = JsonFileUtils()
    # json_file.write('test.json', [{'name': '张三', 'age': 18}, {'name': 'test2', 'age': 20}, {'name': 'test3', 'age': 21}])
    # print(json_file.read('test.json'))

    # yaml_file = YamlFileUtils()
    # yaml_file.write('test.yaml', [{'name': '行数12', 'age': 18}, {'name': 'test2', 'age': 20}, {'name': 'test3', 'age': 22}])
    # print(yaml_file.read('test.yaml'))

    # excel_file = ExcelFileUtils()
    # excel_file.write('test.xls', [['name', 'age'], ['test', 18], ['test2', 20], ['test3', 21]])
    # print(excel_file.read('test.xls'))

    csv_file = CSVFileUtils()
    csv_file.write('test.csv', [['name', 'age'], ['test1', 18], ['test2', 20], ['test3', 20], ['test13', 21]])
    print(csv_file.read('test.csv'))


#!/usr/bin/env python
# coding=utf-8
import os
import pymysql
from dotenv import load_dotenv, find_dotenv
from abc import ABCMeta, abstractmethod
from typing import Union, List, Dict, Tuple, Any
from sshtunnel import SSHTunnelForwarder

class DBUtils(metaclass=ABCMeta):
    def __init__(self, database:str, is_ssh: bool=False, curson_type: bool=True, user_dotenv: bool=True, **kwargs):
        """
        初始化数据库连接
        :param database: 数据库名称
        :param is_ssh: 是否使用SSH隧道
        :param curson_type: 是否使用字典游标
        :param user_dotenv: 是否使用环境变量
        :param kwargs: 其他参数 
        """
        if user_dotenv:
            # 加载环境变量
            load_dotenv(find_dotenv(), override=True)
            # 初始化数据库连接
            self.host = os.getenv("DB_HOST", "localhost")
            self.port = int(os.getenv("DB_PORT", 3306))
            self.user = os.getenv("DB_USER", "root")
            self.password = os.getenv("DB_PASSWORD", "123456")
            self.ssh_host = os.getenv("SSH_HOST", None)
            self.ssh_port = int(os.getenv("SSH_PORT", 22))
            self.ssh_user = os.getenv("SSH_USER", None)
            self.ssh_password = os.getenv("SSH_PASSWORD", None)
        else:
            self.host = kwargs.get("host", "localhost")
            self.port = kwargs.get("port", 3306)
            self.user = kwargs.get("user", "root")
            self.password = kwargs.get("password", "123456")
            if is_ssh:
                self.ssh_host = kwargs.get("ssh_host")
                self.ssh_port = kwargs.get("ssh_port")
                self.ssh_user = kwargs.get("ssh_user")
                self.ssh_password = kwargs.get("ssh_password")
        self.database = database
        self.is_ssh = is_ssh
        self.cursor_type = curson_type

    @abstractmethod
    def connect(self):
        pass

    @abstractmethod
    def execute(self, sql) -> None:
        pass

    @abstractmethod
    def select_one(self) -> Union[None, Dict[str, Any], Tuple[Any]]:
        pass

    @abstractmethod
    def select_all(self) -> Union[None, List[Dict[str, Any]], Tuple[Tuple[Any]]]:
        pass

    @abstractmethod
    def close(self):
        pass


class MysqlUtils(DBUtils):
    def __init__(self, database:str, is_ssh: bool=False, curson_type: bool=True, user_dotenv: bool=True, **kwargs):
        super().__init__(database, is_ssh, curson_type, user_dotenv, **kwargs)

    def connect(self) -> None:
        if self.is_ssh:
            server = SSHTunnelForwarder(
                # 指定ssh登录的跳转机的address
                ssh_address_or_host=(self.ssh_host, int(self.ssh_port)),
                ssh_username=self.ssh_user,
                ssh_password=self.ssh_password,
                # 设置数据库服务地址及端口
                remote_bind_address=(self.host, int(self.port)))
            server.start()
            self.conn = pymysql.connect(
                database=self.database,
                user=self.user,
                password=self.password,
                host='127.0.0.1',
                port=server.local_bind_port,
                cursorclass=pymysql.cursors.DictCursor if self.cursor_type else pymysql.cursors.Cursor
        )
        else:
            self.conn = pymysql.connect(
                host=self.host, port=self.port, user=self.user, password=self.password, database=self.database,
                cursorclass=pymysql.cursors.DictCursor if self.cursor_type else pymysql.cursors.Cursor
        )

    def execute(self, sql) -> None:
        self.cursor = self.conn.cursor()
        self.cursor.execute(sql)

    def select_one(self) -> Union[None, Dict[str, Any], Tuple[Any]]:
        return self.cursor.fetchone()

    def select_all(self) ->  Union[None, List[Dict[str, Any]], Tuple[Tuple[Any]]]:
        return self.cursor.fetchall()

    def close(self) -> None:
        self.conn.close()

def get_db_data(conn: DBUtils, sql: str, is_one: bool = False) -> Union[None, Dict[str, Any], Tuple[Any]]:
    conn.connect()
    conn.execute(sql)
    conn.close()
    if is_one:
        return conn.select_one()
    else:
        return conn.select_all()

if __name__ == '__main__':
    # a = MysqlUtils("base")
    # a.connect()
    # a.execute("select name from user")
    # print(a.select_one())
    # a.close()
    # print(a.select_all())

    # print(get_db_data(a, "select name from user"))
    

    b = MysqlUtils("base", user_dotenv=False)
    print(get_db_data(b, "select name from user"))
    print(get_db_data(b, "select name from user", is_one=True))